@@ -67,8 +67,7 @@ def children(ui, repo, file_=None, **opts):
     See :hg:`help log` and :hg:`help revsets.children`.
 
     """
-    opts = pycompat.byteskwargs(opts)
-    rev = opts.get(b'rev')
+    rev = opts.get('rev')
     ctx = logcmdutil.revsingle(repo, rev)
     if file_:
         fctx = repo.filectx(file_, changeid=ctx.rev())
@@ -76,7 +75,9 @@ def children(ui, repo, file_=None, **opts):
     else:
         childctxs = ctx.children()
 
-    displayer = logcmdutil.changesetdisplayer(ui, repo, opts)
+    displayer = logcmdutil.changesetdisplayer(
+        ui, repo, pycompat.byteskwargs(opts)
+    )
     for cctx in childctxs:
         displayer.show(cctx)
     displayer.close()
