@@ -4,12 +4,11 @@
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
-from __future__ import absolute_import
 
 import os
 
 from mercurial.i18n import _
-from mercurial.node import hex, nullid, nullrev
+from mercurial.node import hex, nullrev
 from mercurial import (
     encoding,
     error,
@@ -33,36 +32,34 @@ from . import (
     shallowutil,
 )
 
-if util.safehasattr(util, '_hgexecutable'):
-    # Before 5be286db
-    _hgexecutable = util.hgexecutable
-else:
-    from mercurial.utils import procutil
-    _hgexecutable = procutil.hgexecutable
-
 # These make*stores functions are global so that other extensions can replace
 # them.
 def makelocalstores(repo):
     """In-repo stores, like .hg/store/data; can not be discarded."""
-    localpath = os.path.join(repo.svfs.vfs.base, 'data')
+    localpath = os.path.join(repo.svfs.vfs.base, b'data')
     if not os.path.exists(localpath):
         os.makedirs(localpath)
 
     # Instantiate local data stores
     localcontent = contentstore.remotefilelogcontentstore(
-        repo, localpath, repo.name, shared=False)
+        repo, localpath, repo.name, shared=False
+    )
     localmetadata = metadatastore.remotefilelogmetadatastore(
-        repo, localpath, repo.name, shared=False)
+        repo, localpath, repo.name, shared=False
+    )
     return localcontent, localmetadata
 
+
 def makecachestores(repo):
     """Typically machine-wide, cache of remote data; can be discarded."""
     # Instantiate shared cache stores
     cachepath = shallowutil.getcachepath(repo.ui)
     cachecontent = contentstore.remotefilelogcontentstore(
-        repo, cachepath, repo.name, shared=True)
+        repo, cachepath, repo.name, shared=True
+    )
     cachemetadata = metadatastore.remotefilelogmetadatastore(
-        repo, cachepath, repo.name, shared=True)
+        repo, cachepath, repo.name, shared=True
+    )
 
     repo.sharedstore = cachecontent
     repo.shareddatastores.append(cachecontent)
@@ -70,30 +67,35 @@ def makecachestores(repo):
 
     return cachecontent, cachemetadata
 
+
 def makeremotestores(repo, cachecontent, cachemetadata):
     """These stores fetch data from a remote server."""
     # Instantiate remote stores
     repo.fileservice = fileserverclient.fileserverclient(repo)
     remotecontent = contentstore.remotecontentstore(
-        repo.ui, repo.fileservice, cachecontent)
+        repo.ui, repo.fileservice, cachecontent
+    )
     remotemetadata = metadatastore.remotemetadatastore(
-        repo.ui, repo.fileservice, cachemetadata)
+        repo.ui, repo.fileservice, cachemetadata
+    )
     return remotecontent, remotemetadata
 
+
 def makepackstores(repo):
     """Packs are more efficient (to read from) cache stores."""
     # Instantiate pack stores
-    packpath = shallowutil.getcachepackpath(repo,
-                                            constants.FILEPACK_CATEGORY)
+    packpath = shallowutil.getcachepackpath(repo, constants.FILEPACK_CATEGORY)
     packcontentstore = datapack.datapackstore(repo.ui, packpath)
     packmetadatastore = historypack.historypackstore(repo.ui, packpath)
 
     repo.shareddatastores.append(packcontentstore)
     repo.sharedhistorystores.append(packmetadatastore)
-    shallowutil.reportpackmetrics(repo.ui, 'filestore', packcontentstore,
-        packmetadatastore)
+    shallowutil.reportpackmetrics(
+        repo.ui, b'filestore', packcontentstore, packmetadatastore
+    )
     return packcontentstore, packmetadatastore
 
+
 def makeunionstores(repo):
     """Union stores iterate the other stores and return the first result."""
     repo.shareddatastores = []
@@ -102,46 +104,66 @@ def makeunionstores(repo):
     packcontentstore, packmetadatastore = makepackstores(repo)
     cachecontent, cachemetadata = makecachestores(repo)
     localcontent, localmetadata = makelocalstores(repo)
-    remotecontent, remotemetadata = makeremotestores(repo, cachecontent,
-                                                     cachemetadata)
+    remotecontent, remotemetadata = makeremotestores(
+        repo, cachecontent, cachemetadata
+    )
 
     # Instantiate union stores
     repo.contentstore = contentstore.unioncontentstore(
-        packcontentstore, cachecontent,
-        localcontent, remotecontent, writestore=localcontent)
+        packcontentstore,
+        cachecontent,
+        localcontent,
+        remotecontent,
+        writestore=localcontent,
+    )
     repo.metadatastore = metadatastore.unionmetadatastore(
-        packmetadatastore, cachemetadata, localmetadata, remotemetadata,
-        writestore=localmetadata)
+        packmetadatastore,
+        cachemetadata,
+        localmetadata,
+        remotemetadata,
+        writestore=localmetadata,
+    )
 
     fileservicedatawrite = cachecontent
     fileservicehistorywrite = cachemetadata
-    repo.fileservice.setstore(repo.contentstore, repo.metadatastore,
-                              fileservicedatawrite, fileservicehistorywrite)
-    shallowutil.reportpackmetrics(repo.ui, 'filestore',
-        packcontentstore, packmetadatastore)
+    repo.fileservice.setstore(
+        repo.contentstore,
+        repo.metadatastore,
+        fileservicedatawrite,
+        fileservicehistorywrite,
+    )
+    shallowutil.reportpackmetrics(
+        repo.ui, b'filestore', packcontentstore, packmetadatastore
+    )
+
 
 def wraprepo(repo):
     class shallowrepository(repo.__class__):
         @util.propertycache
         def name(self):
-            return self.ui.config('remotefilelog', 'reponame')
+            return self.ui.config(b'remotefilelog', b'reponame')
 
         @util.propertycache
         def fallbackpath(self):
-            path = repo.ui.config("remotefilelog", "fallbackpath",
-                                  repo.ui.config('paths', 'default'))
+            path = repo.ui.config(
+                b"remotefilelog",
+                b"fallbackpath",
+                repo.ui.config(b'paths', b'default'),
+            )
             if not path:
-                raise error.Abort("no remotefilelog server "
-                    "configured - is your .hg/hgrc trusted?")
+                raise error.Abort(
+                    b"no remotefilelog server "
+                    b"configured - is your .hg/hgrc trusted?"
+                )
 
             return path
 
         def maybesparsematch(self, *revs, **kwargs):
-            '''
+            """
             A wrapper that allows the remotefilelog to invoke sparsematch() if
             this is a sparse repository, or returns None if this is not a
             sparse repository.
-            '''
+            """
             if revs:
                 ret = sparse.matcher(repo, revs=revs)
             else:
@@ -152,7 +174,7 @@ def wraprepo(repo):
             return ret
 
         def file(self, f):
-            if f[0] == '/':
+            if f[0] == b'/':
                 f = f[1:]
 
             if self.shallowmatch(f):
@@ -164,11 +186,12 @@ def wraprepo(repo):
             if self.shallowmatch(path):
                 return remotefilectx.remotefilectx(self, path, *args, **kwargs)
             else:
-                return super(shallowrepository, self).filectx(path, *args,
-                                                              **kwargs)
+                return super(shallowrepository, self).filectx(
+                    path, *args, **kwargs
+                )
 
         @localrepo.unfilteredmethod
-        def commitctx(self, ctx, error=False):
+        def commitctx(self, ctx, error=False, origctx=None):
             """Add a new revision to current repository.
             Revision information is passed via the context argument.
             """
@@ -181,30 +204,45 @@ def wraprepo(repo):
                 m1 = ctx.p1().manifest()
                 files = []
                 for f in ctx.modified() + ctx.added():
-                    fparent1 = m1.get(f, nullid)
-                    if fparent1 != nullid:
+                    fparent1 = m1.get(f, self.nullid)
+                    if fparent1 != self.nullid:
                         files.append((f, hex(fparent1)))
                 self.fileservice.prefetch(files)
-            return super(shallowrepository, self).commitctx(ctx,
-                                                            error=error)
+            return super(shallowrepository, self).commitctx(
+                ctx, error=error, origctx=origctx
+            )
 
-        def backgroundprefetch(self, revs, base=None, repack=False, pats=None,
-                               opts=None):
-            """Runs prefetch in background with optional repack
-            """
-            cmd = [_hgexecutable(), '-R', repo.origroot, 'prefetch']
+        def backgroundprefetch(
+            self, revs, base=None, repack=False, pats=None, opts=None
+        ):
+            """Runs prefetch in background with optional repack"""
+            cmd = [procutil.hgexecutable(), b'-R', repo.origroot, b'prefetch']
             if repack:
-                cmd.append('--repack')
+                cmd.append(b'--repack')
             if revs:
-                cmd += ['-r', revs]
-            procutil.runbgcommand(cmd, encoding.environ)
+                cmd += [b'-r', revs]
+            # We know this command will find a binary, so don't block
+            # on it starting.
+            kwargs = {}
+            if repo.ui.configbool(b'devel', b'remotefilelog.bg-wait'):
+                kwargs['record_wait'] = repo.ui.atexit
+
+            procutil.runbgcommand(
+                cmd, encoding.environ, ensurestart=False, **kwargs
+            )
 
         def prefetch(self, revs, base=None, pats=None, opts=None):
             """Prefetches all the necessary file revisions for the given revs
             Optionally runs repack in background
             """
-            with repo._lock(repo.svfs, 'prefetchlock', True, None, None,
-                            _('prefetching in %s') % repo.origroot):
+            with repo._lock(
+                repo.svfs,
+                b'prefetchlock',
+                True,
+                None,
+                None,
+                _(b'prefetching in %s') % repo.origroot,
+            ):
                 self._prefetch(revs, base, pats, opts)
 
         def _prefetch(self, revs, base=None, pats=None, opts=None):
@@ -213,20 +251,23 @@ def wraprepo(repo):
                 # If we know a rev is on the server, we should fetch the server
                 # version of those files, since our local file versions might
                 # become obsolete if the local commits are stripped.
-                localrevs = repo.revs('outgoing(%s)', fallbackpath)
+                localrevs = repo.revs(b'outgoing(%s)', fallbackpath)
                 if base is not None and base != nullrev:
-                    serverbase = list(repo.revs('first(reverse(::%s) - %ld)',
-                                                base, localrevs))
+                    serverbase = list(
+                        repo.revs(
+                            b'first(reverse(::%s) - %ld)', base, localrevs
+                        )
+                    )
                     if serverbase:
                         base = serverbase[0]
             else:
                 localrevs = repo
 
             mfl = repo.manifestlog
-            mfrevlog = mfl.getstorage('')
+            mfrevlog = mfl.getstorage(b'')
             if base is not None:
                 mfdict = mfl[repo[base].manifestnode()].read()
-                skip = set(mfdict.iteritems())
+                skip = set(mfdict.items())
             else:
                 skip = set()
 
@@ -237,7 +278,7 @@ def wraprepo(repo):
             visited = set()
             visited.add(nullrev)
             revcount = len(revs)
-            progress = self.ui.makeprogress(_('prefetching'), total=revcount)
+            progress = self.ui.makeprogress(_(b'prefetching'), total=revcount)
             progress.update(0)
             for rev in sorted(revs):
                 ctx = repo[rev]
@@ -256,7 +297,7 @@ def wraprepo(repo):
                 else:
                     mfdict = mfl[mfnode].read()
 
-                diff = mfdict.iteritems()
+                diff = mfdict.items()
                 if pats:
                     diff = (pf for pf in diff if m(pf[0]))
                 if sparsematch:
@@ -293,13 +334,16 @@ def wraprepo(repo):
 
     makeunionstores(repo)
 
-    repo.includepattern = repo.ui.configlist("remotefilelog", "includepattern",
-                                             None)
-    repo.excludepattern = repo.ui.configlist("remotefilelog", "excludepattern",
-                                             None)
-    if not util.safehasattr(repo, 'connectionpool'):
+    repo.includepattern = repo.ui.configlist(
+        b"remotefilelog", b"includepattern", None
+    )
+    repo.excludepattern = repo.ui.configlist(
+        b"remotefilelog", b"excludepattern", None
+    )
+    if not hasattr(repo, 'connectionpool'):
         repo.connectionpool = connectionpool.connectionpool(repo)
 
     if repo.includepattern or repo.excludepattern:
-        repo.shallowmatch = match.match(repo.root, '', None,
-            repo.includepattern, repo.excludepattern)
+        repo.shallowmatch = match.match(
+            repo.root, b'', None, repo.includepattern, repo.excludepattern
+        )
