@@ -24,7 +24,8 @@ remotenames.hoistedpeer
   namespace (default: 'default')
 """
 
-from __future__ import absolute_import
+
+import collections.abc
 
 from mercurial.i18n import _
 
@@ -35,7 +36,6 @@ from mercurial import (
     extensions,
     logexchange,
     namespaces,
-    pycompat,
     registrar,
     revsetlang,
     smartset,
@@ -45,15 +45,6 @@ from mercurial import (
 
 from mercurial.utils import stringutil
 
-if pycompat.ispy3:
-    import collections.abc
-
-    mutablemapping = collections.abc.MutableMapping
-else:
-    import collections
-
-    mutablemapping = collections.MutableMapping
-
 # Note for extension authors: ONLY specify testedwith = 'ships-with-hg-core' for
 # extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
 # be specifying the version(s) of Mercurial they are tested with, or
@@ -82,7 +73,7 @@ configitem(
 )
 
 
-class lazyremotenamedict(mutablemapping):
+class lazyremotenamedict(collections.abc.MutableMapping):
     """
     Read-only dict-like Class to lazily resolve remotename entries
 
@@ -102,7 +93,7 @@ class lazyremotenamedict(mutablemapping):
         self.loaded = False
 
     def _load(self):
-        """ Read the remotenames file, store entries matching selected kind """
+        """Read the remotenames file, store entries matching selected kind"""
         self.loaded = True
         repo = self._repo
         for node, rpath, rname in logexchange.readremotenamefile(
@@ -112,7 +103,7 @@ class lazyremotenamedict(mutablemapping):
             self.potentialentries[name] = (node, rpath, name)
 
     def _resolvedata(self, potentialentry):
-        """ Check that the node for potentialentry exists and return it """
+        """Check that the node for potentialentry exists and return it"""
         if not potentialentry in self.potentialentries:
             return None
         node, remote, name = self.potentialentries[potentialentry]
@@ -160,24 +151,24 @@ class lazyremotenamedict(mutablemapping):
             return None
 
     def keys(self):
-        """ Get a list of bookmark or branch names """
+        """Get a list of bookmark or branch names"""
         if not self.loaded:
             self._load()
         return self.potentialentries.keys()
 
     def iteritems(self):
-        """ Iterate over (name, node) tuples """
+        """Iterate over (name, node) tuples"""
 
         if not self.loaded:
             self._load()
 
-        for k, vtup in pycompat.iteritems(self.potentialentries):
+        for k, vtup in self.potentialentries.items():
             yield (k, [bin(vtup[0])])
 
     items = iteritems
 
 
-class remotenames(object):
+class remotenames:
     """
     This class encapsulates all the remotenames state. It also contains
     methods to access that state in convenient ways. Remotenames are lazy
@@ -190,7 +181,7 @@ class remotenames(object):
         self.clearnames()
 
     def clearnames(self):
-        """ Clear all remote names state """
+        """Clear all remote names state"""
         self.bookmarks = lazyremotenamedict(b"bookmarks", self._repo)
         self.branches = lazyremotenamedict(b"branches", self._repo)
         self._invalidatecache()
@@ -208,7 +199,7 @@ class remotenames(object):
         if not self._nodetobmarks:
             bmarktonodes = self.bmarktonodes()
             self._nodetobmarks = {}
-            for name, node in pycompat.iteritems(bmarktonodes):
+            for name, node in bmarktonodes.items():
                 self._nodetobmarks.setdefault(node[0], []).append(name)
         return self._nodetobmarks
 
@@ -219,7 +210,7 @@ class remotenames(object):
         if not self._nodetobranch:
             branchtonodes = self.branchtonodes()
             self._nodetobranch = {}
-            for name, nodes in pycompat.iteritems(branchtonodes):
+            for name, nodes in branchtonodes.items():
                 for node in nodes:
                     self._nodetobranch.setdefault(node, []).append(name)
         return self._nodetobranch
@@ -229,7 +220,7 @@ class remotenames(object):
             marktonodes = self.bmarktonodes()
             self._hoisttonodes = {}
             hoist += b'/'
-            for name, node in pycompat.iteritems(marktonodes):
+            for name, node in marktonodes.items():
                 if name.startswith(hoist):
                     name = name[len(hoist) :]
                     self._hoisttonodes[name] = node
@@ -240,7 +231,7 @@ class remotenames(object):
             marktonodes = self.bmarktonodes()
             self._nodetohoists = {}
             hoist += b'/'
-            for name, node in pycompat.iteritems(marktonodes):
+            for name, node in marktonodes.items():
                 if name.startswith(hoist):
                     name = name[len(hoist) :]
                     self._nodetohoists.setdefault(node[0], []).append(name)
@@ -264,7 +255,7 @@ def wrapprintbookmarks(orig, ui, repo, fm, bmarks):
 
 
 def extsetup(ui):
-    extensions.wrapfunction(bookmarks, b'_printbookmarks', wrapprintbookmarks)
+    extensions.wrapfunction(bookmarks, '_printbookmarks', wrapprintbookmarks)
 
 
 def reposetup(ui, repo):
