@@ -39,9 +39,7 @@ Config
   skiphash = False
 """
 
-from __future__ import absolute_import
 
-import hashlib
 import inspect
 import os
 import re
@@ -51,40 +49,48 @@ import struct
 import time
 
 from .i18n import _
+from .node import hex
 
 from . import (
     commandserver,
     encoding,
     error,
     extensions,
-    node,
     pycompat,
     util,
 )
 
 from .utils import (
+    hashutil,
     procutil,
     stringutil,
 )
 
+
 def _hashlist(items):
     """return sha1 hexdigest for a list"""
-    return node.hex(hashlib.sha1(stringutil.pprint(items)).digest())
+    return hex(hashutil.sha1(stringutil.pprint(items)).digest())
+
 
 # sensitive config sections affecting confighash
 _configsections = [
-    'alias',  # affects global state commands.table
-    'eol',    # uses setconfig('eol', ...)
-    'extdiff',  # uisetup will register new commands
-    'extensions',
+    b'alias',  # affects global state commands.table
+    b'diff-tools',  # affects whether gui or not in extdiff's uisetup
+    b'eol',  # uses setconfig('eol', ...)
+    b'extdiff',  # uisetup will register new commands
+    b'extensions',
+    b'fastannotate',  # affects annotate command and adds fastannonate cmd
+    b'merge-tools',  # affects whether gui or not in extdiff's uisetup
+    b'schemes',  # extsetup will update global hg.schemes
 ]
 
 _configsectionitems = [
-    ('commands', 'show.aliasprefix'), # show.py reads it in extsetup
+    (b'commands', b'show.aliasprefix'),  # show.py reads it in extsetup
 ]
 
 # sensitive environment variables affecting confighash
-_envre = re.compile(br'''\A(?:
+_envre = re.compile(
+    br'''\A(?:
                     CHGHG
                     |HG(?:DEMANDIMPORT|EMITWARNINGS|MODULEPOLICY|PROF|RCPATH)?
                     |HG(?:ENCODING|PLAIN).*
@@ -95,7 +101,10 @@ _envre = re.compile(br'''\A(?:
                     |PYTHON.*
                     |TERM(?:INFO)?
                     |TZ
-                    )\Z''', re.X)
+                    )\Z''',
+    re.X,
+)
+
 
 def _confighash(ui):
     """return a quick hash for detecting config/env changes
@@ -115,15 +124,19 @@ def _confighash(ui):
         sectionitems.append(ui.config(section, item))
     sectionhash = _hashlist(sectionitems)
     # If $CHGHG is set, the change to $HG should not trigger a new chg server
-    if 'CHGHG' in encoding.environ:
-        ignored = {'HG'}
+    if b'CHGHG' in encoding.environ:
+        ignored = {b'HG'}
     else:
         ignored = set()
-    envitems = [(k, v) for k, v in encoding.environ.iteritems()
-                if _envre.match(k) and k not in ignored]
+    envitems = [
+        (k, v)
+        for k, v in encoding.environ.items()
+        if _envre.match(k) and k not in ignored
+    ]
     envhash = _hashlist(sorted(envitems))
     return sectionhash[:6] + envhash[:6]
 
+
 def _getmtimepaths(ui):
     """get a list of paths that should be checked to detect change
 
@@ -135,10 +148,13 @@ def _getmtimepaths(ui):
     modules = [m for n, m in extensions.extensions(ui)]
     try:
         from . import __version__
+
         modules.append(__version__)
     except ImportError:
         pass
-    files = [pycompat.sysexecutable]
+    files = []
+    if pycompat.sysexecutable:
+        files.append(pycompat.sysexecutable)
     for m in modules:
         try:
             files.append(pycompat.fsencode(inspect.getabsfile(m)))
@@ -146,6 +162,7 @@ def _getmtimepaths(ui):
             pass
     return sorted(set(files))
 
+
 def _mtimehash(paths):
     """return a quick hash for detecting file changes
 
@@ -163,6 +180,7 @@ def _mtimehash(paths):
     extensions after importing them (there is imp.find_module but that faces
     race conditions). We need to calculate confighash without importing.
     """
+
     def trystat(path):
         try:
             st = os.stat(path)
@@ -170,10 +188,13 @@ def _mtimehash(paths):
         except OSError:
             # could be ENOENT, EPERM etc. not fatal in any case
             pass
-    return _hashlist(map(trystat, paths))[:12]
 
-class hashstate(object):
+    return _hashlist(pycompat.maplist(trystat, paths))[:12]
+
+
+class hashstate:
     """a structure storing confighash, mtimehash, paths used for mtimehash"""
+
     def __init__(self, confighash, mtimehash, mtimepaths):
         self.confighash = confighash
         self.mtimehash = mtimehash
@@ -185,10 +206,15 @@ class hashstate(object):
             mtimepaths = _getmtimepaths(ui)
         confighash = _confighash(ui)
         mtimehash = _mtimehash(mtimepaths)
-        ui.log('cmdserver', 'confighash = %s mtimehash = %s\n',
-               confighash, mtimehash)
+        ui.log(
+            b'cmdserver',
+            b'confighash = %s mtimehash = %s\n',
+            confighash,
+            mtimehash,
+        )
         return hashstate(confighash, mtimehash, mtimepaths)
 
+
 def _newchgui(srcui, csystem, attachio):
     class chgui(srcui.__class__):
         def __init__(self, src=None):
@@ -204,46 +230,53 @@ def _newchgui(srcui, csystem, attachio):
             #  b. or stdout is redirected by protectfinout(),
             # because the chg client is not aware of these situations and
             # will behave differently (i.e. write to stdout).
-            if (out is not self.fout
-                or not util.safehasattr(self.fout, 'fileno')
+            if (
+                out is not self.fout
+                or not hasattr(self.fout, 'fileno')
                 or self.fout.fileno() != procutil.stdout.fileno()
-                or self._finoutredirected):
+                or self._finoutredirected
+            ):
                 return procutil.system(cmd, environ=environ, cwd=cwd, out=out)
             self.flush()
             return self._csystem(cmd, procutil.shellenviron(environ), cwd)
 
         def _runpager(self, cmd, env=None):
-            self._csystem(cmd, procutil.shellenviron(env), type='pager',
-                          cmdtable={'attachio': attachio})
+            self._csystem(
+                cmd,
+                procutil.shellenviron(env),
+                type=b'pager',
+                cmdtable={b'attachio': attachio},
+            )
             return True
 
     return chgui(srcui)
 
+
 def _loadnewui(srcui, args, cdebug):
     from . import dispatch  # avoid cycle
 
     newui = srcui.__class__.load()
     for a in ['fin', 'fout', 'ferr', 'environ']:
         setattr(newui, a, getattr(srcui, a))
-    if util.safehasattr(srcui, '_csystem'):
+    if hasattr(srcui, '_csystem'):
         newui._csystem = srcui._csystem
 
     # command line args
     options = dispatch._earlyparseopts(newui, args)
-    dispatch._parseconfig(newui, options['config'])
+    dispatch._parseconfig(newui, options[b'config'])
 
     # stolen from tortoisehg.util.copydynamicconfig()
     for section, name, value in srcui.walkconfig():
         source = srcui.configsource(section, name)
-        if ':' in source or source == '--config' or source.startswith('$'):
+        if b':' in source or source == b'--config' or source.startswith(b'$'):
             # path:line or command line, or environ
             continue
         newui.setconfig(section, name, value, source)
 
     # load wd and repo config, copied from dispatch.py
-    cwd = options['cwd']
+    cwd = options[b'cwd']
     cwd = cwd and os.path.realpath(cwd) or None
-    rpath = options['repository']
+    rpath = options[b'repository']
     path, newlui = dispatch._getlocal(newui, rpath, wd=cwd)
 
     extensions.populateui(newui)
@@ -254,7 +287,8 @@ def _loadnewui(srcui, args, cdebug):
 
     return (newui, newlui)
 
-class channeledsystem(object):
+
+class channeledsystem:
     """Propagate ui.system() request in the following format:
 
     payload length (unsigned int),
@@ -274,27 +308,28 @@ class channeledsystem(object):
     and executes it defined by cmdtable, or exits the loop if the command name
     is empty.
     """
+
     def __init__(self, in_, out, channel):
         self.in_ = in_
         self.out = out
         self.channel = channel
 
-    def __call__(self, cmd, environ, cwd=None, type='system', cmdtable=None):
-        args = [type, procutil.quotecommand(cmd), os.path.abspath(cwd or '.')]
-        args.extend('%s=%s' % (k, v) for k, v in environ.iteritems())
-        data = '\0'.join(args)
-        self.out.write(struct.pack('>cI', self.channel, len(data)))
+    def __call__(self, cmd, environ, cwd=None, type=b'system', cmdtable=None):
+        args = [type, cmd, util.abspath(cwd or b'.')]
+        args.extend(b'%s=%s' % (k, v) for k, v in environ.items())
+        data = b'\0'.join(args)
+        self.out.write(struct.pack(b'>cI', self.channel, len(data)))
         self.out.write(data)
         self.out.flush()
 
-        if type == 'system':
+        if type == b'system':
             length = self.in_.read(4)
-            length, = struct.unpack('>I', length)
+            (length,) = struct.unpack(b'>I', length)
             if length != 4:
-                raise error.Abort(_('invalid response'))
-            rc, = struct.unpack('>i', self.in_.read(4))
+                raise error.Abort(_(b'invalid response'))
+            (rc,) = struct.unpack(b'>i', self.in_.read(4))
             return rc
-        elif type == 'pager':
+        elif type == b'pager':
             while True:
                 cmd = self.in_.readline()[:-1]
                 if not cmd:
@@ -302,23 +337,30 @@ class channeledsystem(object):
                 if cmdtable and cmd in cmdtable:
                     cmdtable[cmd]()
                 else:
-                    raise error.Abort(_('unexpected command: %s') % cmd)
+                    raise error.Abort(_(b'unexpected command: %s') % cmd)
         else:
-            raise error.ProgrammingError('invalid S channel type: %s' % type)
+            raise error.ProgrammingError(b'invalid S channel type: %s' % type)
+
 
 _iochannels = [
     # server.ch, ui.fp, mode
-    ('cin', 'fin', r'rb'),
-    ('cout', 'fout', r'wb'),
-    ('cerr', 'ferr', r'wb'),
+    ('cin', 'fin', 'rb'),
+    ('cout', 'fout', 'wb'),
+    ('cerr', 'ferr', 'wb'),
 ]
 
+
 class chgcmdserver(commandserver.server):
-    def __init__(self, ui, repo, fin, fout, sock, prereposetups,
-                 hashstate, baseaddress):
+    def __init__(
+        self, ui, repo, fin, fout, sock, prereposetups, hashstate, baseaddress
+    ):
         super(chgcmdserver, self).__init__(
-            _newchgui(ui, channeledsystem(fin, fout, 'S'), self.attachio),
-            repo, fin, fout, prereposetups)
+            _newchgui(ui, channeledsystem(fin, fout, b'S'), self.attachio),
+            repo,
+            fin,
+            fout,
+            prereposetups,
+        )
         self.clientsock = sock
         self._ioattached = False
         self._oldios = []  # original (self.ch, ui.fp, fd) before "attachio"
@@ -326,7 +368,7 @@ class chgcmdserver(commandserver.server):
         self.baseaddress = baseaddress
         if hashstate is not None:
             self.capabilities = self.capabilities.copy()
-            self.capabilities['validate'] = chgcmdserver.validate
+            self.capabilities[b'validate'] = chgcmdserver.validate
 
     def cleanup(self):
         super(chgcmdserver, self).cleanup()
@@ -342,9 +384,19 @@ class chgcmdserver(commandserver.server):
         """
         # tell client to sendmsg() with 1-byte payload, which makes it
         # distinctive from "attachio\n" command consumed by client.read()
-        self.clientsock.sendall(struct.pack('>cI', 'I', 1))
-        clientfds = util.recvfds(self.clientsock.fileno())
-        self.ui.log('chgserver', 'received fds: %r\n', clientfds)
+        self.clientsock.sendall(struct.pack(b'>cI', b'I', 1))
+
+        data, ancdata, msg_flags, address = self.clientsock.recvmsg(1, 256)
+        assert len(ancdata) == 1
+        cmsg_level, cmsg_type, cmsg_data = ancdata[0]
+        assert cmsg_level == socket.SOL_SOCKET
+        assert cmsg_type == socket.SCM_RIGHTS
+        # memoryview.cast() was added in typeshed 61600d68772a, but pytype
+        # still complains
+        # pytype: disable=attribute-error
+        clientfds = memoryview(cmsg_data).cast('i').tolist()
+        # pytype: enable=attribute-error
+        self.ui.log(b'chgserver', b'received fds: %r\n', clientfds)
 
         ui = self.ui
         ui.flush()
@@ -360,21 +412,21 @@ class chgcmdserver(commandserver.server):
             # to see output immediately on pager, the mode stays unchanged
             # when client re-attached. ferr is unchanged because it should
             # be unbuffered no matter if it is a tty or not.
-            if fn == 'ferr':
+            if fn == b'ferr':
                 newfp = fp
             else:
-                # make it line buffered explicitly because the default is
-                # decided on first write(), where fout could be a pager.
+                # On Python 3, the standard library doesn't offer line-buffered
+                # binary streams, so wrap/unwrap it.
                 if fp.isatty():
-                    bufsize = 1  # line buffered
+                    newfp = procutil.make_line_buffered(fp)
                 else:
-                    bufsize = -1  # system default
-                newfp = os.fdopen(fp.fileno(), mode, bufsize)
+                    newfp = procutil.unwrap_line_buffered(fp)
+            if newfp is not fp:
                 setattr(ui, fn, newfp)
             setattr(self, cn, newfp)
 
         self._ioattached = True
-        self.cresult.write(struct.pack('>i', len(clientfds)))
+        self.cresult.write(struct.pack(b'>i', len(clientfds)))
 
     def _saveio(self):
         if self._oldios:
@@ -387,18 +439,35 @@ class chgcmdserver(commandserver.server):
             self._oldios.append((ch, fp, fd))
 
     def _restoreio(self):
+        if not self._oldios:
+            return
+        nullfd = os.open(os.devnull, os.O_WRONLY)
         ui = self.ui
-        for (ch, fp, fd), (cn, fn, _mode) in zip(self._oldios, _iochannels):
-            newfp = getattr(ui, fn)
-            # close newfp while it's associated with client; otherwise it
-            # would be closed when newfp is deleted
-            if newfp is not fp:
-                newfp.close()
-            # restore original fd: fp is open again
-            os.dup2(fd, fp.fileno())
-            os.close(fd)
+        for (ch, fp, fd), (cn, fn, mode) in zip(self._oldios, _iochannels):
+            try:
+                if 'w' in mode:
+                    # Discard buffered data which couldn't be flushed because
+                    # of EPIPE. The data should belong to the current session
+                    # and should never persist.
+                    os.dup2(nullfd, fp.fileno())
+                    fp.flush()
+                os.dup2(fd, fp.fileno())
+                os.close(fd)
+            except OSError as err:
+                # According to issue6330, running chg on heavy loaded systems
+                # can lead to EBUSY. [man dup2] indicates that, on Linux,
+                # EBUSY comes from a race condition between open() and dup2().
+                # However it's not clear why open() race occurred for
+                # newfd=stdin/out/err.
+                self.ui.log(
+                    b'chgserver',
+                    b'got %s while duplicating %s\n',
+                    stringutil.forcebytestr(err),
+                    fn,
+                )
             setattr(self, cn, ch)
             setattr(ui, fn, fp)
+        os.close(nullfd)
         del self._oldios[:]
 
     def validate(self):
@@ -421,37 +490,45 @@ class chgcmdserver(commandserver.server):
         list, the client can continue with this server after completing all
         the instructions.
         """
-        from . import dispatch  # avoid cycle
-
         args = self._readlist()
+        errorraised = False
+        detailed_exit_code = 255
         try:
             self.ui, lui = _loadnewui(self.ui, args, self.cdebug)
-        except error.ParseError as inst:
-            dispatch._formatparse(self.ui.warn, inst)
-            self.ui.flush()
-            self.cresult.write('exit 255')
-            return
-        except error.Abort as inst:
-            self.ui.error(_("abort: %s\n") % inst)
+        except error.RepoError as inst:
+            # RepoError can be raised while trying to read shared source
+            # configuration
+            self.ui.error(_(b"abort: %s\n") % stringutil.forcebytestr(inst))
             if inst.hint:
-                self.ui.error(_("(%s)\n") % inst.hint)
+                self.ui.error(_(b"(%s)\n") % inst.hint)
+            errorraised = True
+        except error.Error as inst:
+            if inst.detailed_exit_code is not None:
+                detailed_exit_code = inst.detailed_exit_code
+            self.ui.error(inst.format())
+            errorraised = True
+
+        if errorraised:
             self.ui.flush()
-            self.cresult.write('exit 255')
+            exit_code = 255
+            if self.ui.configbool(b'ui', b'detailed-exit-code'):
+                exit_code = detailed_exit_code
+            self.cresult.write(b'exit %d' % exit_code)
             return
         newhash = hashstate.fromui(lui, self.hashstate.mtimepaths)
         insts = []
         if newhash.mtimehash != self.hashstate.mtimehash:
             addr = _hashaddress(self.baseaddress, self.hashstate.confighash)
-            insts.append('unlink %s' % addr)
+            insts.append(b'unlink %s' % addr)
             # mtimehash is empty if one or more extensions fail to load.
             # to be compatible with hg, still serve the client this time.
             if self.hashstate.mtimehash:
-                insts.append('reconnect')
+                insts.append(b'reconnect')
         if newhash.confighash != self.hashstate.confighash:
             addr = _hashaddress(self.baseaddress, newhash.confighash)
-            insts.append('redirect %s' % addr)
-        self.ui.log('chgserver', 'validate: %s\n', stringutil.pprint(insts))
-        self.cresult.write('\0'.join(insts) or '\0')
+            insts.append(b'redirect %s' % addr)
+        self.ui.log(b'chgserver', b'validate: %s\n', stringutil.pprint(insts))
+        self.cresult.write(b'\0'.join(insts) or b'\0')
 
     def chdir(self):
         """Change current directory
@@ -462,7 +539,7 @@ class chgcmdserver(commandserver.server):
         path = self._readstr()
         if not path:
             return
-        self.ui.log('chgserver', 'chdir to %r\n', path)
+        self.ui.log(b'chgserver', b"chdir to '%s'\n", path)
         os.chdir(path)
 
     def setumask(self):
@@ -475,13 +552,13 @@ class chgcmdserver(commandserver.server):
         """Change umask"""
         data = self._readstr()
         if len(data) != 4:
-            raise ValueError('invalid mask length in setumask2 request')
+            raise ValueError(b'invalid mask length in setumask2 request')
         self._setumask(data)
 
     def _setumask(self, data):
-        mask = struct.unpack('>I', data)[0]
-        self.ui.log('chgserver', 'setumask %r\n', mask)
-        os.umask(mask)
+        mask = struct.unpack(b'>I', data)[0]
+        self.ui.log(b'chgserver', b'setumask %r\n', mask)
+        util.setumask(mask)
 
     def runcommand(self):
         # pager may be attached within the runcommand session, which should
@@ -502,48 +579,65 @@ class chgcmdserver(commandserver.server):
         """
         l = self._readlist()
         try:
-            newenv = dict(s.split('=', 1) for s in l)
+            newenv = dict(s.split(b'=', 1) for s in l)
         except ValueError:
-            raise ValueError('unexpected value in setenv request')
-        self.ui.log('chgserver', 'setenv: %r\n', sorted(newenv.keys()))
+            raise ValueError(b'unexpected value in setenv request')
+        self.ui.log(b'chgserver', b'setenv: %r\n', sorted(newenv.keys()))
+
         encoding.environ.clear()
         encoding.environ.update(newenv)
 
     capabilities = commandserver.server.capabilities.copy()
-    capabilities.update({'attachio': attachio,
-                         'chdir': chdir,
-                         'runcommand': runcommand,
-                         'setenv': setenv,
-                         'setumask': setumask,
-                         'setumask2': setumask2})
+    capabilities.update(
+        {
+            b'attachio': attachio,
+            b'chdir': chdir,
+            b'runcommand': runcommand,
+            b'setenv': setenv,
+            b'setumask': setumask,
+            b'setumask2': setumask2,
+        }
+    )
+
+    if hasattr(procutil, 'setprocname'):
 
-    if util.safehasattr(procutil, 'setprocname'):
         def setprocname(self):
             """Change process title"""
             name = self._readstr()
-            self.ui.log('chgserver', 'setprocname: %r\n', name)
+            self.ui.log(b'chgserver', b'setprocname: %r\n', name)
             procutil.setprocname(name)
-        capabilities['setprocname'] = setprocname
+
+        capabilities[b'setprocname'] = setprocname
+
 
 def _tempaddress(address):
-    return '%s.%d.tmp' % (address, os.getpid())
+    return b'%s.%d.tmp' % (address, os.getpid())
+
 
 def _hashaddress(address, hashstr):
     # if the basename of address contains '.', use only the left part. this
     # makes it possible for the client to pass 'server.tmp$PID' and follow by
     # an atomic rename to avoid locking when spawning new servers.
     dirname, basename = os.path.split(address)
-    basename = basename.split('.', 1)[0]
-    return '%s-%s' % (os.path.join(dirname, basename), hashstr)
+    basename = basename.split(b'.', 1)[0]
+    return b'%s-%s' % (os.path.join(dirname, basename), hashstr)
 
-class chgunixservicehandler(object):
+
+class chgunixservicehandler:
     """Set of operations for chg services"""
 
     pollinterval = 1  # [sec]
 
     def __init__(self, ui):
         self.ui = ui
-        self._idletimeout = ui.configint('chgserver', 'idletimeout')
+
+        # TODO: use PEP 526 syntax (`_hashstate: hashstate` at the class level)
+        #  when 3.5 support is dropped.
+        self._hashstate = None  # type: hashstate
+        self._baseaddress = None  # type: bytes
+        self._realaddress = None  # type: bytes
+
+        self._idletimeout = ui.configint(b'chgserver', b'idletimeout')
         self._lastactive = time.time()
 
     def bindsocket(self, sock, address):
@@ -555,7 +649,7 @@ class chgunixservicehandler(object):
 
     def _inithashstate(self, address):
         self._baseaddress = address
-        if self.ui.configbool('chgserver', 'skiphash'):
+        if self.ui.configbool(b'chgserver', b'skiphash'):
             self._hashstate = None
             self._realaddress = address
             return
@@ -569,7 +663,7 @@ class chgunixservicehandler(object):
             # one or more extensions failed to load. mtimehash becomes
             # meaningless because we do not know the paths of those extensions.
             # set mtimehash to an illegal hash value to invalidate the server.
-            self._hashstate.mtimehash = ''
+            self._hashstate.mtimehash = b''
 
     def _bind(self, sock):
         # use a unique temp address so we can stat the file and do ownership
@@ -592,8 +686,10 @@ class chgunixservicehandler(object):
     def _issocketowner(self):
         try:
             st = os.stat(self._realaddress)
-            return (st.st_ino == self._socketstat.st_ino and
-                    st[stat.ST_MTIME] == self._socketstat[stat.ST_MTIME])
+            return (
+                st.st_ino == self._socketstat.st_ino
+                and st[stat.ST_MTIME] == self._socketstat[stat.ST_MTIME]
+            )
         except OSError:
             return False
 
@@ -608,8 +704,9 @@ class chgunixservicehandler(object):
 
     def shouldexit(self):
         if not self._issocketowner():
-            self.ui.log(b'chgserver', b'%s is not owned, exiting.\n',
-                        self._realaddress)
+            self.ui.log(
+                b'chgserver', b'%s is not owned, exiting.\n', self._realaddress
+            )
             return True
         if time.time() - self._lastactive > self._idletimeout:
             self.ui.log(b'chgserver', b'being idle too long. exiting.\n')
@@ -620,8 +717,17 @@ class chgunixservicehandler(object):
         self._lastactive = time.time()
 
     def createcmdserver(self, repo, conn, fin, fout, prereposetups):
-        return chgcmdserver(self.ui, repo, fin, fout, conn, prereposetups,
-                            self._hashstate, self._baseaddress)
+        return chgcmdserver(
+            self.ui,
+            repo,
+            fin,
+            fout,
+            conn,
+            prereposetups,
+            self._hashstate,
+            self._baseaddress,
+        )
+
 
 def chgunixservice(ui, repo, opts):
     # CHGINTERNALMARK is set by chg client. It is an indication of things are
@@ -629,11 +735,21 @@ def chgunixservice(ui, repo, opts):
     # demandimport or detecting chg client started by chg client. When executed
     # here, CHGINTERNALMARK is no longer useful and hence dropped to make
     # environ cleaner.
-    if 'CHGINTERNALMARK' in encoding.environ:
-        del encoding.environ['CHGINTERNALMARK']
+    if b'CHGINTERNALMARK' in encoding.environ:
+        del encoding.environ[b'CHGINTERNALMARK']
+    # Python3.7+ "coerces" the LC_CTYPE environment variable to a UTF-8 one if
+    # it thinks the current value is "C". This breaks the hash computation and
+    # causes chg to restart loop.
+    if b'CHGORIG_LC_CTYPE' in encoding.environ:
+        encoding.environ[b'LC_CTYPE'] = encoding.environ[b'CHGORIG_LC_CTYPE']
+        del encoding.environ[b'CHGORIG_LC_CTYPE']
+    elif b'CHG_CLEAR_LC_CTYPE' in encoding.environ:
+        if b'LC_CTYPE' in encoding.environ:
+            del encoding.environ[b'LC_CTYPE']
+        del encoding.environ[b'CHG_CLEAR_LC_CTYPE']
 
     if repo:
         # one chgserver can serve multiple repos. drop repo information
-        ui.setconfig('bundle', 'mainreporoot', '', 'repo')
+        ui.setconfig(b'bundle', b'mainreporoot', b'', b'repo')
     h = chgunixservicehandler(ui)
     return commandserver.unixforkingservice(ui, repo=None, opts=opts, handler=h)
