@@ -20,7 +20,6 @@ import time
 
 from .i18n import _
 from .pycompat import (
-    getattr,
     open,
 )
 from . import (
@@ -54,9 +53,9 @@ class STARTTLS(smtplib.SMTP):
         self._ui = ui
         self._host = host
 
-    def starttls(self, keyfile=None, certfile=None):
+    def starttls(self, keyfile=None, certfile=None, context=None):
         if not self.has_extn("starttls"):
-            msg = b"STARTTLS extension not supported by server"
+            msg = "STARTTLS extension not supported by server"
             raise smtplib.SMTPException(msg)
         (resp, reply) = self.docmd("STARTTLS")
         if resp == 220:
