@@ -203,6 +203,13 @@ class bytestr(bytes):
     >>> bytestr(bytesable())
     'bytes'
 
+    ...unless the argument is the bytes *type* itself: it gets a
+    __bytes__() method in Python 3.11, which cannot be used as in an instance
+    of bytes:
+
+    >>> bytestr(bytes)
+    "<class 'bytes'>"
+
     There's no implicit conversion from non-ascii str as its encoding is
     unknown:
 
@@ -252,10 +259,9 @@ class bytestr(bytes):
     def __new__(cls: Type[_Tbytestr], s: object = b'') -> _Tbytestr:
         if isinstance(s, bytestr):
             return s
-        if not isinstance(
-            s, (bytes, bytearray)
-        ) and not builtins.hasattr(  # hasattr-py3-only
-            s, u'__bytes__'
+        if not isinstance(s, (bytes, bytearray)) and (
+            isinstance(s, type)
+            or not builtins.hasattr(s, u'__bytes__')  # hasattr-py3-only
         ):
             s = str(s).encode('ascii')
         return bytes.__new__(cls, s)
@@ -355,6 +361,13 @@ def getdoc(obj: object) -> Optional[bytes]:
 def _wrapattrfunc(f):
     @functools.wraps(f)
     def w(object, name, *args):
+        if isinstance(name, bytes):
+            from . import util
+
+            msg = b'function "%s" take `str` as argument, not `bytes`'
+            fname = f.__name__.encode('ascii')
+            msg %= fname
+            util.nouideprecwarn(msg, b"6.6", stacklevel=2)
         return f(object, sysstr(name), *args)
 
     return w
