@@ -6,7 +6,6 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
 
 import copy
 import weakref
@@ -16,11 +15,6 @@ from .node import (
     hex,
     nullrev,
 )
-from .pycompat import (
-    delattr,
-    getattr,
-    setattr,
-)
 from . import (
     error,
     obsolete,
@@ -48,8 +42,7 @@ def hideablerevs(repo):
 
 
 def pinnedrevs(repo):
-    """revisions blocking hidden changesets from being filtered
-    """
+    """revisions blocking hidden changesets from being filtered"""
 
     cl = repo.changelog
     pinned = set()
@@ -62,6 +55,19 @@ def pinnedrevs(repo):
         rev = cl.index.get_rev
         pinned.update(rev(t[0]) for t in tags.values())
         pinned.discard(None)
+
+    # Avoid cycle: mercurial.filemerge -> mercurial.templater ->
+    # mercurial.templatefuncs -> mercurial.revset -> mercurial.repoview ->
+    # mercurial.mergestate -> mercurial.filemerge
+    from . import mergestate
+
+    ms = mergestate.mergestate.read(repo)
+    if ms.active() and ms.unresolvedcount():
+        for node in (ms.local, ms.other):
+            rev = cl.index.get_rev(node)
+            if rev is not None:
+                pinned.add(rev)
+
     return pinned
 
 
@@ -147,7 +153,7 @@ def computeimpactable(repo, visibilityexceptions=None):
     This filter out any mutable changeset and any public changeset that may be
     impacted by something happening to a mutable revision.
 
-    This is achieved by filtered everything with a revision number egal or
+    This is achieved by filtered everything with a revision number equal or
     higher than the first mutable changeset is filtered."""
     assert not repo.changelog.filteredrevs
     cl = repo.changelog
@@ -157,7 +163,7 @@ def computeimpactable(repo, visibilityexceptions=None):
         firstmutable = min(firstmutable, min(cl.rev(r) for r in roots))
     # protect from nullrev root
     firstmutable = max(0, firstmutable)
-    return frozenset(pycompat.xrange(firstmutable, len(cl)))
+    return frozenset(range(firstmutable, len(cl)))
 
 
 # function to compute filtered set
@@ -198,16 +204,18 @@ def extrafilter(ui):
     subsettable = repoviewutil.subsettable
 
     if combine(b'base') not in filtertable:
-        for name in _basefiltername:
+        for base_name in _basefiltername:
 
-            def extrafilteredrevs(repo, *args, **kwargs):
+            def extrafilteredrevs(repo, *args, name=base_name, **kwargs):
                 baserevs = filtertable[name](repo, *args, **kwargs)
                 extrarevs = frozenset(repo.revs(frevs))
                 return baserevs | extrarevs
 
-            filtertable[combine(name)] = extrafilteredrevs
-            if name in subsettable:
-                subsettable[combine(name)] = combine(subsettable[name])
+            filtertable[combine(base_name)] = extrafilteredrevs
+            if base_name in subsettable:
+                subsettable[combine(base_name)] = combine(
+                    subsettable[base_name]
+                )
     return fid
 
 
@@ -250,10 +258,10 @@ def wrapchangelog(unfichangelog, filteredrevs):
     return cl
 
 
-class filteredchangelogmixin(object):
+class filteredchangelogmixin:
     def tiprev(self):
         """filtered version of revlog.tiprev"""
-        for i in pycompat.xrange(len(self) - 1, -2, -1):
+        for i in range(len(self) - 1, -2, -1):
             if i not in self.filteredrevs:
                 return i
 
@@ -265,7 +273,7 @@ class filteredchangelogmixin(object):
         """filtered version of revlog.__iter__"""
 
         def filterediter():
-            for i in pycompat.xrange(len(self)):
+            for i in range(len(self)):
                 if i not in self.filteredrevs:
                     yield i
 
@@ -283,13 +291,12 @@ class filteredchangelogmixin(object):
         This returns a version of 'revs' to be used thereafter by the caller.
         In particular, if revs is an iterator, it is converted into a set.
         """
-        safehasattr = util.safehasattr
-        if safehasattr(revs, '__next__'):
+        if hasattr(revs, '__next__'):
             # Note that inspect.isgenerator() is not true for iterators,
             revs = set(revs)
 
         filteredrevs = self.filteredrevs
-        if safehasattr(revs, 'first'):  # smartset
+        if hasattr(revs, 'first'):  # smartset
             offenders = revs & filteredrevs
         else:
             offenders = filteredrevs.intersection(revs)
@@ -321,7 +328,7 @@ class filteredchangelogmixin(object):
         r = super(filteredchangelogmixin, self).rev(node)
         if r in self.filteredrevs:
             raise error.FilteredLookupError(
-                hex(node), self.indexfile, _(b'filtered node')
+                hex(node), self.display_id, _(b'filtered node')
             )
         return r
 
@@ -350,7 +357,7 @@ class filteredchangelogmixin(object):
         return super(filteredchangelogmixin, self).flags(rev)
 
 
-class repoview(object):
+class repoview:
     """Provide a read/write view of a repo through a filtered changelog
 
     This object is used to access a filtered version of a repository without
@@ -409,7 +416,7 @@ class repoview(object):
         with util.timedcm('repo filter for %s', self.filtername):
             revs = filterrevs(unfi, self.filtername, self._visibilityexceptions)
         cl = self._clcache
-        newkey = (unfilen, unfinode, hash(revs), unfichangelog._delayed)
+        newkey = (unfilen, unfinode, hash(revs), unfichangelog.is_delaying)
         # if cl.index is not unfiindex, unfi.changelog would be
         # recreated, and our clcache refers to garbage object
         if cl is not None and (
@@ -452,18 +459,24 @@ class repoview(object):
         return delattr(self._unfilteredrepo, attr)
 
 
-# Python <3.4 easily leaks types via __mro__. See
-# https://bugs.python.org/issue17950. We cache dynamically created types
-# so they won't be leaked on every invocation of repo.filtered().
+# Dynamically created classes introduce memory cycles via __mro__. See
+# https://bugs.python.org/issue17950.
+# This need of the garbage collector can turn into memory leak in
+# Python <3.4, which is the first version released with PEP 442.
 _filteredrepotypes = weakref.WeakKeyDictionary()
 
 
 def newtype(base):
     """Create a new type with the repoview mixin and the given base class"""
-    if base not in _filteredrepotypes:
+    ref = _filteredrepotypes.get(base)
+    if ref is not None:
+        cls = ref()
+        if cls is not None:
+            return cls
 
-        class filteredrepo(repoview, base):
-            pass
+    class filteredrepo(repoview, base):
+        pass
 
-        _filteredrepotypes[base] = filteredrepo
-    return _filteredrepotypes[base]
+    _filteredrepotypes[base] = weakref.ref(filteredrepo)
+    # do not reread from weakref to be 100% sure not to return None
+    return filteredrepo
